<?php

namespace Drupal\rrssb\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Class RRSSBSettingsForm.
 *
 * @package Drupal\rrssb\Form
 */
class RRSSBSettingsForm extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [
      'rrssb.settings',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'rrssb_settings';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $all_buttons = rrssb_settings(TRUE);
    $config = $this->config('rrssb.settings');
    $chosen = $config->get('chosen');
    $form['follow'] = array(
      '#type' => 'select',
      '#title' => $this->t('Select type of buttons'),
      '#options' => array(
        0 => $this->t('Share'),
        1 => $this->t('Follow'),
      ),
      '#default_value' => $config->get('follow'),
      '#description' => $this->t('"Share" buttons invite the visitor to share the page from your site onto their page/channel/profile.  "Follow" buttons direct the visitor to your page/channel/profile.'),
    );

    // Create the config for the table of buttons.
    $form['chosen'] = [
      '#type' => 'table',
      '#header' => [
        $this->t('Button'),
        $this->t('Enabled'),
        $this->t('Username'),
        $this->t('Weight'),
      ],
      '#empty' => $this->t('No buttons found'),
      '#tabledrag' => [
        [
          'action' => 'order',
          'relationship' => 'sibling',
          'group' => 'item-row-weight',
        ],
      ],
    ];

    foreach ($all_buttons as $name => $button) {
      $form['chosen'][$name]['#attributes']['class'][] = 'draggable';

      // Determine if this button requires a particular value of follow
      // to be valid. This is the case if one or other of the URL as not
      // present.
      // Both URLs absent makes no sense and would be a bug.
      unset($require_follow);
      if (!isset($button['follow_url'])) {
        $require_follow = 0;
      }
      elseif (!isset($button['share_url'])) {
        $require_follow = 1;
      }
      $form['chosen'][$name]['label'] = array(
        '#type' => 'item',
        '#markup' => $name,
      );
      $form['chosen'][$name]['enabled'] = array(
        '#type' => 'checkbox',
        '#default_value' => isset($chosen[$name]['enabled']) ? $chosen[$name]['enabled'] : FALSE,
      );
      if (isset($require_follow)) {
        // Hide entries where there is no corresponding URL.
        $form['chosen'][$name]['enabled']['#states'] = array(
          'visible' => array(":input[name='follow']" => array('value' => $require_follow)),
        );
      }
      if (isset($button['follow_url'])) {
        $form['chosen'][$name]['username'] = array(
          '#type' => 'textfield',
          '#default_value' => isset($chosen[$name]['username']) ? $chosen[$name]['username'] : '',
          // Hide the username for share URLs where it isn't needed.
          // Otherwise it is a required field.
          '#states' => array(
            'visible' => array(":input[name='follow']" => array('value' => 1)),
            'required' => array(
              ":input[name='follow']" => array('value' => 1),
              ":input[name='chosen[$name][enabled]']" => array('checked' => TRUE),
            ),
          ),
        );
      }
      else {
        $form['chosen'][$name]['username'] = array();
      }
      $form['chosen'][$name]['weight'] = array(
        '#type' => 'weight',
        '#title' => $this->t('Weight for @title', array('@title' => $name)),
        '#title_display' => 'invisible',
        '#default_value' => isset($chosen[$name]['weight']) ? $chosen[$name]['weight'] : 0,
        '#delta' => 20,
        '#attributes' => array('class' => array('item-row-weight')),
      );
    }

    // Appearance settings stored as an array ready to pass to the library code.
    $form['appearance'] = array(
      '#type' => 'fieldset',
      '#tree' => TRUE,
      '#title' => $this->t('Appearance'),
    );
    $form['appearance']['size'] = array(
      '#type' => 'textfield',
      '#title' => $this->t('Size'),
      '#size' => 5,
      '#default_value' => $config->get('appearance.size'),
      '#description' => $this->t('Size, as a proportion of default size set in CSS.'),
    );
    $form['appearance']['shrink'] = array(
      '#type' => 'textfield',
      '#title' => $this->t('Minimum size'),
      '#size' => 5,
      '#default_value' => $config->get('appearance.shrink'),
      '#description' => $this->t('Minimum size to shrink buttons to, as a proportion of original size.'),
    );
    $form['appearance']['regrow'] = array(
      '#type' => 'textfield',
      '#title' => $this->t('Extra row size'),
      '#size' => 5,
      '#default_value' => $config->get('appearance.regrow'),
      '#description' => $this->t('Maximum size of buttons after they have been forced to split onto extra rows of buttons, as a proportion of original size.'),
    );
    $form['appearance']['minRows'] = array(
      '#type' => 'textfield',
      '#title' => $this->t('Minimum rows'),
      '#size' => 5,
      '#default_value' => $config->get('appearance.minRows'),
      '#description' => $this->t('Minimum number of rows of buttons.  Set to a large value to create vertical layout.'),
    );
    $form['appearance']['maxRows'] = array(
      '#type' => 'textfield',
      '#title' => $this->t('Maximum rows'),
      '#size' => 5,
      '#default_value' => $config->get('appearance.maxRows'),
      '#description' => $this->t('Maximum number of rows of buttons.  If more rows would be needed, instead the labels are hidden.  Set to a large value to keep labels if at all possible.'),
    );
    $form['appearance']['prefixReserve'] = array(
      '#type' => 'textfield',
      '#title' => $this->t('Prefix reserved width'),
      '#size' => 5,
      '#default_value' => $config->get('appearance.prefixReserve'),
      '#description' => $this->t('Proportion of total width reserved for prefix to be inline.'),
    );
    $form['appearance']['prefixHide'] = array(
      '#type' => 'textfield',
      '#title' => $this->t('Prefix maximum width'),
      '#size' => 5,
      '#default_value' => $config->get('appearance.prefixHide'),
      '#description' => $this->t('Maximum prefix width as a proportion of total width before hiding prefix.'),
    );

    $form['prefix'] = array(
      '#type' => 'textfield',
      '#title' => $this->t('Prefix text before the buttons'),
      '#default_value' => $config->get('prefix'),
      '#description' => $this->t('Put this text before the buttons.  For example "Follow us" or "Share this page".'),
    );
    $form['image_tokens'] = array(
      '#type' => 'textfield',
      '#title' => $this->t('Tokens to use to find images'),
      '#default_value' => $config->get('image_tokens'),
      '#description' => $this->t('Enter one or more tokens, separated by |.  These tokens will be tried in turn to determine the image to use in buttons.
        The default value is [node:field_image]|[rrssbsite:logo-url] which you can adapt to pick other fields or as desired.'),
    );
    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    parent::validateForm($form, $form_state);
    $values = $form_state->getValues();

    foreach ($values['chosen'] as $name => $settings) {
      if ($settings['enabled']) {
        if ($values['follow'] && isset($settings['username']) && !$settings['username']) {
          form_set_error("chosen[$name][username]", t('You must set the username to use "Follow" button for !button', array('!button' => $name)));
        }
        // If a button is enabled where there is no URL, we don't count that
        // as an error, just don't show the button (@see rrssb_settings).
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    parent::submitForm($form, $form_state);
    $config = $this->config('rrssb.settings');
    $values = $form_state->cleanValues()->getValues();

    // Write config.
    foreach ($values as $key => $value) {
      $config->set($key, $value);
    }

    $config->save();

    // Clear cache and run our associated local hooks.
    \Drupal::cache()->invalidate('rrssb_buttons');
    rrssb_flush_caches();

    //@@ Need to clear the cached block and the node field?
  }
}
